@@ -335,10 +335,7 @@ where
         E: Encoder,
     {
         match self {
-            Some(value) => {
-                let encoder = encoder.encode_some()?;
-                value.encode(encoder)
-            }
+            Some(value) => encoder.encode_some(|encoder| value.encode(encoder)),
             None => encoder.encode_none(),
         }
     }
@@ -371,20 +368,20 @@ where
     where
         E: Encoder,
     {
-        let mut variant = encoder.encode_struct_variant(1)?;
+        encoder.encode_struct_variant(1, |mut variant| {
+            match self {
+                Ok(ok) => {
+                    Encode::encode(&0usize, variant.first()?)?;
+                    Encode::encode(ok, variant.second()?)?;
+                }
+                Err(err) => {
+                    Encode::encode(&1usize, variant.first()?)?;
+                    Encode::encode(err, variant.second()?)?;
+                }
+            }
 
-        match self {
-            Ok(ok) => {
-                Encode::encode(&0usize, variant.first()?)?;
-                Encode::encode(ok, variant.second()?)?;
-                variant.end()
-            }
-            Err(err) => {
-                Encode::encode(&1usize, variant.first()?)?;
-                Encode::encode(err, variant.second()?)?;
-                variant.end()
-            }
-        }
+            variant.end()
+        })
     }
 }
 
@@ -398,13 +395,13 @@ where
     where
         D: Decoder<'de>,
     {
-        let mut variant = decoder.decode_variant()?;
-
-        match variant.first().and_then(usize::decode)? {
-            0 => variant.second().and_then(T::decode).map(Ok),
-            1 => variant.second().and_then(U::decode).map(Err),
-            tag => Err(D::Error::invalid_variant_tag("Result", tag)),
-        }
+        decoder.decode_variant(
+            |mut variant| match variant.first().and_then(usize::decode)? {
+                0 => variant.second().and_then(T::decode).map(Ok),
+                1 => variant.second().and_then(U::decode).map(Err),
+                tag => Err(D::Error::invalid_variant_tag("Result", tag)),
+            },
+        )
     }
 }
 
